@@ -3,7 +3,7 @@ from enthought.chaco.api import ArrayDataSource, DataRange1D, LinearMapper,BarPl
                                  add_default_grids,VPlotContainer
 from enthought.chaco.tools.api import PanTool, ZoomTool,RangeSelection,RangeSelectionOverlay
 from enthought.chaco.api import create_line_plot
-from enthought.traits.ui.api import View,Item,VGroup
+from enthought.traits.ui.api import View,Item,VGroup,HGroup
 from enthought.traits.api import HasTraits,DelegatesTo,Trait
 from enthought.traits.api import Float, Instance, Int,Bool,Str,Unicode,Enum,Button
 from enthought.chaco.api import AbstractOverlay, BaseXYPlot
@@ -21,9 +21,9 @@ from numpy import array, compress, column_stack, invert, isnan, transpose, zeros
 from enthought.traits.api import List
 from enthought.enable.colors import ColorTrait
 from enthought.pyface.timer import timer
+
 process_colors=[0x000000,0x555555,0xffff88,0x55ffff,0xAD2D2D, 0xeeeeee,0xeeaaaa,0xaaaaee,0xee0000]
 class TimeChartOptions(HasTraits):
-    minimum_time_filter = Enum((0,1000,10000,50000,100000,500000,1000000,5000000,1000000,5000000,10000000,50000000))
     remove_pids_not_on_screen = Bool(True)
     show_wake_events = Bool(False)
     show_p_states = Bool(True)
@@ -32,15 +32,6 @@ class TimeChartOptions(HasTraits):
 
     proj = tcProject
 
-    traits_view = View(VGroup(
-            Item('minimum_time_filter'),
-            Item('remove_pids_not_on_screen'),
-            Item('show_wake_events'),
-            Item('show_p_states'),
-            Item('show_c_states'),
-            Item('auto_zoom_y'),
-            label='Display Properties'
-            ))
     def connect(self,plot):
         self.auto_zoom_timer = timer.Timer(300,self._auto_zoom_y_delayed)
         self.auto_zoom_timer.Stop()
@@ -61,26 +52,45 @@ class TimeChartOptions(HasTraits):
     def _auto_zoom_y_delayed(self):
         self.plot.auto_zoom_y()
         self.auto_zoom_timer.Stop()
+    def _on_toggle_autohide(self, value):
+        self.remove_pids_not_on_screen = value
+    def _on_toggle_wakes(self, value):
+        self.show_wake_events = value
+    def _on_toggle_cpuidle(self, value):
+        self.show_c_states = value
+    def _on_toggle_cpufreq(self, value):
+        self.show_p_states = value
+    def _on_toggle_auto_zoom_y(self, value):
+        self.show_auto_zoom_y = value
+
 class TextView(HasTraits):
     text = Str
+    save = Button()
     def __init__(self,text,title):
         self.text = text
         self.trait_view().title = title
 
     traits_view = View(
-            Item('text',style="custom",show_label=False),
-            resizable = True,
-            width = 1024,
-            height = 600,
-            )
+        Item('text',style="custom",show_label=False),
+        HGroup(
+            Item('save'),
+            show_labels = False),
+        resizable = True,
+        width = 1024,
+        height = 600,
+        )
+    def _save_changed(self):
+        from window import save_dialog
+        fn = save_dialog()
+        if fn:
+            try:
+                f = open(fn,"w")
+                f.write(self.text)
+                f.close()
+            except:
+                print "unable to write file..."
 class RangeSelectionTools(HasTraits):
     time = Str
-    c_states = Str
-    traits_view = View(VGroup(
-            Item('time'),
-            Item('c_states',style="custom"),
-            label='Selection Infos'
-            ))
     start = 0
     end = 0
     def connect(self,plot):
@@ -93,37 +103,33 @@ class RangeSelectionTools(HasTraits):
             self.start, self.end = amin(value), amax(value)
             time = self.end-self.start
             self.time = "%d.%03d %03ds"%(time/1000000,(time/1000)%1000,time%1000)
+            self.plot.immediate_invalidate()
+            self._timer.Stop()
             self._timer.Start()
+        else:
+            self.start = 0
+            self.end = 0
     def _on_zoom(self):
-        self.plot.index_range.high = self.end
-        self.plot.index_range.low = self.start
-        self.plot.range_selection.deselect()
-        self.plot.invalidate_draw()
-        self.plot.request_redraw()
+        if self.end != self.start:
+            self.plot.index_range.high = self.end
+            self.plot.index_range.low = self.start
+            self.plot.range_selection.deselect()
+            self.plot.invalidate_draw()
+            self.plot.request_redraw()
     def _on_unzoom(self):
         self.plot.index_range.high = self.plot.highest_i
         self.plot.index_range.low = self.plot.lowest_i
         self.plot.invalidate_draw()
         self.plot.request_redraw()
     def _on_trace_text(self):
-        text = self.plot.proj.get_selection_text(self.start,self.end)
-        text_view = TextView(text,"%s:[%d:%d]"%(self.plot.proj.filename,self.start,self.end))
-        text_view.edit_traits()
+        if self.end != self.start:
+            text = self.plot.proj.get_selection_text(self.start,self.end)
+            text_view = TextView(text,"%s:[%d:%d]"%(self.plot.proj.filename,self.start,self.end))
+            text_view.edit_traits()
 
     def _selection_updated_delayed(self):
-        c_states_stats = self.plot.proj.c_states_stats(self.start,self.end)
-        tmp = ""
-        i=0
-        for cpu_stat in c_states_stats:
-            tmp+="cpu%d:\n"%(i)
-            i+=1
-            for cstate in sorted([i for i in cpu_stat.keys()]):
-                part = cpu_stat[cstate]
-                tmp += "C%d:%dus %02.f%%\n"%(cstate,part,part*100/(self.end-self.start))
-        self.c_states = tmp
         self.plot.proj.process_stats(self.start,self.end)
         self._timer.Stop()
-        pass
 class tcPlot(BarPlot):
     """custom plot to draw the timechart
     probably not very 'chacotic' We draw the chart as a whole
@@ -146,6 +152,9 @@ class tcPlot(BarPlot):
     def invalidate(self):
         self.invalidate_draw()
         self.request_redraw()
+    def immediate_invalidate(self):
+        self.invalidate_draw()
+        self.request_redraw_delayed()
 
     def request_redraw_delayed(self):
         self.redraw_timer.Stop()
@@ -182,7 +191,6 @@ class tcPlot(BarPlot):
         gc.translate_ctm(*(-offset))
         return l_w,l_h
     def _draw_timechart(self,gc,tc,label,base_y):
-        
         bar_middle_y = self.first_bar_y+(base_y+.5)*self.bar_height
         points = self._gather_timechart_points(tc.start_ts,tc.end_ts,base_y,.2)
         if self.options.remove_pids_not_on_screen and points.size == 0:
@@ -196,7 +204,7 @@ class tcPlot(BarPlot):
             l_w,l_h = self._draw_label(gc,label,tc.name,self.x,bar_middle_y)
             self.last_label = bar_middle_y-8
         else:
-            l_w,l_h = 0,0 
+            l_w,l_h = 0,0
         if points.size != 0:
             # draw the middle line from end of label to end of screen
             if l_w != 0: # we did not draw label because too short on space
@@ -252,16 +260,13 @@ class tcPlot(BarPlot):
                     if points.size>0:
                         # map the bars start and stop locations into screen space
                         gc.set_alpha(1)
-                        lower_left_pts = self.map_screen(points[:,(0,2)]) 
+                        lower_left_pts = self.map_screen(points[:,(0,2)])
                         upper_right_pts = self.map_screen(points[:,(1,3)])
                         bounds = upper_right_pts - lower_left_pts
                         rects=column_stack((lower_left_pts, bounds))
                         gc.rects(rects)
                         gc.draw_path()
-                
-                    
         return 1
-            
     def _draw_freqchart(self,gc,tc,label,y):
         self._draw_bg(gc,y,tc.bg_color)
         low_i = searchsorted(tc.start_ts,self.index_mapper.range.low)
@@ -271,10 +276,8 @@ class tcPlot(BarPlot):
             low_i -=1
         if high_i<len(tc.start_ts):
             high_i +=1
-        
         if low_i>=high_i-1:
             return array([])
-        
         start_ts = tc.start_ts[low_i:high_i-1]
         end_ts = tc.start_ts[low_i+1:high_i]
         values = (tc.types[low_i:high_i-1]/(float(tc.max_types)))+y
@@ -366,8 +369,6 @@ class tcPlot(BarPlot):
         not_on_screen = []
         on_screen = []
         for tc in self.proj.processes:
-            if tc.total_time < self.options.minimum_time_filter:
-                continue
             if tc.show==False:
                 continue
             processes_y[(tc.comm,tc.pid)] = y+.5
@@ -431,15 +432,16 @@ def create_timechart_container(project):
     plot.lowest_i = low
     plot.highest_i = high
     project.on_trait_change(plot.invalidate, "plot_redraw")
+    project.on_trait_change(plot.invalidate, "selected")
     max_process = 50
     if value_range.high>max_process:
         value_range.low = value_range.high-max_process
-    # Attach some tools 
-    plot.tools.append(PanTool(plot,drag_button='left'))
+    # Attach some tools
+    plot.tools.append(tools.myPanTool(plot,drag_button='left'))
     zoom = tools.myZoomTool(component=plot, tool_mode="range", always_on=True,axis="index",drag_button=None)
     plot.tools.append(zoom)
 
-    plot.range_selection = RangeSelection(plot,resize_margin=1)
+    plot.range_selection = tools.myRangeSelection(plot,resize_margin=3)
     plot.tools.append(plot.range_selection)
     plot.overlays.append(RangeSelectionOverlay(component=plot,axis="index",use_backbuffer=True))
 
